import{s as ye,n as Te,o as ve}from"../chunks/scheduler.9039eef2.js";import{S as Pe,i as Me,g as r,s,r as o,A as Le,h as i,f as a,c as l,j as be,u as m,x as h,k as we,y as Ce,a as n,v as p,d,t as u,w as c}from"../chunks/index.ded8f90d.js";import{C as oe}from"../chunks/CodeBlock.8580f3e8.js";import{H as B}from"../chunks/Heading.f027f30d.js";function qe(me){let f,U,A,V,g,X,$,pe="Here we present an approach that uses a single base model for the entire PPO algorithm - which includes retrieving the reference logits, computing the active logits and the rewards. This feature is experimental as we did not test the convergence of the approach. We encourage the community to let us know if they potentially face issues.",O,_,S,b,de="You just need to install <code>peft</code> and optionally install <code>bitsandbytes</code> as well if you want to go for 8bit base models, for more memory efficient finetuning.",E,w,I,y,ue="You need to address this approach in three stages that we summarize as follows:",Z,T,ce=`1- Train a base model on the target domain (e.g. <code>imdb</code> dataset) - this is the Supervised Fine Tuning stage - it can leverage the <code>SFTTrainer</code> from TRL.
2- Train a reward model using <code>peft</code>. This is required in order to re-use the adapter during the RL optimisation process (step 3 below). We show an example of leveraging the <code>RewardTrainer</code> from TRL in <a href="https://github.com/huggingface/trl/tree/main/examples/scripts/reward_modeling.py" rel="nofollow">this example</a>
3- Fine tune new adapters on the base model using PPO and the reward adapter. (“0 abstraction RL”)`,Y,v,fe="Make sure to use the same model (i.e. same architecture and same weights) for the stages 2 & 3.",z,P,G,M,he=`Let us assume you have trained your reward adapter on <code>llama-7b</code> model using <code>RewardTrainer</code> and pushed the weights on the hub under <code>trl-lib/llama-7b-hh-rm-adapter</code>.
When doing PPO, before passing the model to <code>PPOTrainer</code> create your model as follows:`,N,L,Q,C,ge="Then inside your PPO training loop, call the <code>compute_reward_score</code> method by accessing the <code>model</code> attribute from <code>PPOTrainer</code>.",K,q,D,x,ee,H,te,j,$e=`If you are familiar with the <code>peft</code> library, you know that you can use multiple adapters inside the same model. What you can do is train multiple adapters on the same base model to fine-tune on different policies.
In this case, you want to be able to control the adapter name you want to activate back, after retrieving the reward. For that, simply pass the appropriate <code>adapter_name</code> to <code>ppo_adapter_name</code> argument when calling <code>compute_reward_score</code>.`,ae,R,ne,k,se,J,_e=`For more memory efficient fine-tuning, you can load your base model in 8-bit or 4-bit while keeping the adapters in the default precision (float32).
Just pass the appropriate arguments (i.e. <code>load_in_8bit=True</code> or <code>load_in_4bit=True</code>) to <code>AutoModelForCausalLMWithValueHead.from_pretrained</code> as follows (assuming you have installed <code>bitsandbytes</code>):`,le,W,re,F,ie;return g=new B({props:{title:"Multi Adapter RL (MARL) - a single base model for everything",local:"multi-adapter-rl-marl---a-single-base-model-for-everything",headingTag:"h1"}}),_=new B({props:{title:"Requirements",local:"requirements",headingTag:"h2"}}),w=new B({props:{title:"Summary",local:"summary",headingTag:"h2"}}),P=new B({props:{title:"Quickstart",local:"quickstart",headingTag:"h2"}}),L=new oe({props:{code:"JTBBJTBBJTBB",highlighted:`model_name = <span class="hljs-string">&quot;huggyllama/llama-7b&quot;</span>
rm_adapter_id = <span class="hljs-string">&quot;trl-lib/llama-7b-hh-rm-adapter&quot;</span>

<span class="hljs-comment"># PPO adapter</span>
lora_config = LoraConfig(
    r=<span class="hljs-number">16</span>,
    lora_alpha=<span class="hljs-number">32</span>,
    lora_dropout=<span class="hljs-number">0.05</span>,
    bias=<span class="hljs-string">&quot;none&quot;</span>,
    task_type=<span class="hljs-string">&quot;CAUSAL_LM&quot;</span>,
)

model = AutoModelForCausalLMWithValueHead.from_pretrained(
    model_name,
    peft_config=lora_config,
    reward_adapter=rm_adapter_id,
)

...
trainer = PPOTrainer(
    model=model,
    ...
)

...`,wrap:!1}}),q=new oe({props:{code:"cmV3YXJkcyUyMCUzRCUyMHRyYWluZXIubW9kZWwuY29tcHV0ZV9yZXdhcmRfc2NvcmUoKippbnB1dHMp",highlighted:"rewards = trainer.model.compute_reward_score(**inputs)",wrap:!1}}),x=new B({props:{title:"Advanced usage",local:"advanced-usage",headingTag:"h2"}}),H=new B({props:{title:"Control on the adapter name",local:"control-on-the-adapter-name",headingTag:"h3"}}),R=new oe({props:{code:"YWRhcHRlcl9uYW1lX3BvbGljeV8xJTIwJTNEJTIwJTIycG9saWN5XzElMjIlMEFyZXdhcmRzJTIwJTNEJTIwdHJhaW5lci5tb2RlbC5jb21wdXRlX3Jld2FyZF9zY29yZSgqKmlucHV0cyUyQyUyMHBwb19hZGFwdGVyX25hbWUlM0RhZGFwdGVyX25hbWVfcG9saWN5XzEpJTBBLi4u",highlighted:`adapter_name_policy_1 = <span class="hljs-string">&quot;policy_1&quot;</span>
rewards = trainer.model.compute_reward_score(**inputs, ppo_adapter_name=adapter_name_policy_1)
...`,wrap:!1}}),k=new B({props:{title:"Using 4-bit and 8-bit base models",local:"using-4-bit-and-8-bit-base-models",headingTag:"h3"}}),W=new oe({props:{code:"JTBBJTBB",highlighted:`model_name = <span class="hljs-string">&quot;llama-7b&quot;</span>
rm_adapter_id = <span class="hljs-string">&quot;trl-lib/llama-7b-hh-rm-adapter&quot;</span>

<span class="hljs-comment"># PPO adapter</span>
lora_config = LoraConfig(
    r=<span class="hljs-number">16</span>,
    lora_alpha=<span class="hljs-number">32</span>,
    lora_dropout=<span class="hljs-number">0.05</span>,
    bias=<span class="hljs-string">&quot;none&quot;</span>,
    task_type=<span class="hljs-string">&quot;CAUSAL_LM&quot;</span>,
)

model = AutoModelForCausalLMWithValueHead.from_pretrained(
    model_name,
    peft_config=lora_config,
    reward_adapter=rm_adapter_id,
    load_in_8bit=<span class="hljs-literal">True</span>,
)

...
trainer = PPOTrainer(
    model=model,
    ...
)
...`,wrap:!1}}),{c(){f=r("meta"),U=s(),A=r("p"),V=s(),o(g.$$.fragment),X=s(),$=r("p"),$.textContent=pe,O=s(),o(_.$$.fragment),S=s(),b=r("p"),b.innerHTML=de,E=s(),o(w.$$.fragment),I=s(),y=r("p"),y.textContent=ue,Z=s(),T=r("p"),T.innerHTML=ce,Y=s(),v=r("p"),v.textContent=fe,z=s(),o(P.$$.fragment),G=s(),M=r("p"),M.innerHTML=he,N=s(),o(L.$$.fragment),Q=s(),C=r("p"),C.innerHTML=ge,K=s(),o(q.$$.fragment),D=s(),o(x.$$.fragment),ee=s(),o(H.$$.fragment),te=s(),j=r("p"),j.innerHTML=$e,ae=s(),o(R.$$.fragment),ne=s(),o(k.$$.fragment),se=s(),J=r("p"),J.innerHTML=_e,le=s(),o(W.$$.fragment),re=s(),F=r("p"),this.h()},l(e){const t=Le("svelte-u9bgzb",document.head);f=i(t,"META",{name:!0,content:!0}),t.forEach(a),U=l(e),A=i(e,"P",{}),be(A).forEach(a),V=l(e),m(g.$$.fragment,e),X=l(e),$=i(e,"P",{"data-svelte-h":!0}),h($)!=="svelte-1ni6bkb"&&($.textContent=pe),O=l(e),m(_.$$.fragment,e),S=l(e),b=i(e,"P",{"data-svelte-h":!0}),h(b)!=="svelte-1crdfuc"&&(b.innerHTML=de),E=l(e),m(w.$$.fragment,e),I=l(e),y=i(e,"P",{"data-svelte-h":!0}),h(y)!=="svelte-1nxlg56"&&(y.textContent=ue),Z=l(e),T=i(e,"P",{"data-svelte-h":!0}),h(T)!=="svelte-b8wlu0"&&(T.innerHTML=ce),Y=l(e),v=i(e,"P",{"data-svelte-h":!0}),h(v)!=="svelte-x4geit"&&(v.textContent=fe),z=l(e),m(P.$$.fragment,e),G=l(e),M=i(e,"P",{"data-svelte-h":!0}),h(M)!=="svelte-yusdbc"&&(M.innerHTML=he),N=l(e),m(L.$$.fragment,e),Q=l(e),C=i(e,"P",{"data-svelte-h":!0}),h(C)!=="svelte-uwecsv"&&(C.innerHTML=ge),K=l(e),m(q.$$.fragment,e),D=l(e),m(x.$$.fragment,e),ee=l(e),m(H.$$.fragment,e),te=l(e),j=i(e,"P",{"data-svelte-h":!0}),h(j)!=="svelte-3l2jwi"&&(j.innerHTML=$e),ae=l(e),m(R.$$.fragment,e),ne=l(e),m(k.$$.fragment,e),se=l(e),J=i(e,"P",{"data-svelte-h":!0}),h(J)!=="svelte-17i0flk"&&(J.innerHTML=_e),le=l(e),m(W.$$.fragment,e),re=l(e),F=i(e,"P",{}),be(F).forEach(a),this.h()},h(){we(f,"name","hf:doc:metadata"),we(f,"content",xe)},m(e,t){Ce(document.head,f),n(e,U,t),n(e,A,t),n(e,V,t),p(g,e,t),n(e,X,t),n(e,$,t),n(e,O,t),p(_,e,t),n(e,S,t),n(e,b,t),n(e,E,t),p(w,e,t),n(e,I,t),n(e,y,t),n(e,Z,t),n(e,T,t),n(e,Y,t),n(e,v,t),n(e,z,t),p(P,e,t),n(e,G,t),n(e,M,t),n(e,N,t),p(L,e,t),n(e,Q,t),n(e,C,t),n(e,K,t),p(q,e,t),n(e,D,t),p(x,e,t),n(e,ee,t),p(H,e,t),n(e,te,t),n(e,j,t),n(e,ae,t),p(R,e,t),n(e,ne,t),p(k,e,t),n(e,se,t),n(e,J,t),n(e,le,t),p(W,e,t),n(e,re,t),n(e,F,t),ie=!0},p:Te,i(e){ie||(d(g.$$.fragment,e),d(_.$$.fragment,e),d(w.$$.fragment,e),d(P.$$.fragment,e),d(L.$$.fragment,e),d(q.$$.fragment,e),d(x.$$.fragment,e),d(H.$$.fragment,e),d(R.$$.fragment,e),d(k.$$.fragment,e),d(W.$$.fragment,e),ie=!0)},o(e){u(g.$$.fragment,e),u(_.$$.fragment,e),u(w.$$.fragment,e),u(P.$$.fragment,e),u(L.$$.fragment,e),u(q.$$.fragment,e),u(x.$$.fragment,e),u(H.$$.fragment,e),u(R.$$.fragment,e),u(k.$$.fragment,e),u(W.$$.fragment,e),ie=!1},d(e){e&&(a(U),a(A),a(V),a(X),a($),a(O),a(S),a(b),a(E),a(I),a(y),a(Z),a(T),a(Y),a(v),a(z),a(G),a(M),a(N),a(Q),a(C),a(K),a(D),a(ee),a(te),a(j),a(ae),a(ne),a(se),a(J),a(le),a(re),a(F)),a(f),c(g,e),c(_,e),c(w,e),c(P,e),c(L,e),c(q,e),c(x,e),c(H,e),c(R,e),c(k,e),c(W,e)}}}const xe='{"title":"Multi Adapter RL (MARL) - a single base model for everything","local":"multi-adapter-rl-marl---a-single-base-model-for-everything","sections":[{"title":"Requirements","local":"requirements","sections":[],"depth":2},{"title":"Summary","local":"summary","sections":[],"depth":2},{"title":"Quickstart","local":"quickstart","sections":[],"depth":2},{"title":"Advanced usage","local":"advanced-usage","sections":[{"title":"Control on the adapter name","local":"control-on-the-adapter-name","sections":[],"depth":3},{"title":"Using 4-bit and 8-bit base models","local":"using-4-bit-and-8-bit-base-models","sections":[],"depth":3}],"depth":2}],"depth":1}';function He(me){return ve(()=>{new URLSearchParams(window.location.search).get("fw")}),[]}class We extends Pe{constructor(f){super(),Me(this,f,He,qe,ye,{})}}export{We as component};
